package riomhaire.lti;

import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.lang.NonNull;
import org.springframework.web.servlet.ModelAndView;
import riomhaire.lti.core.business.LTIMessageDispatcher;
import riomhaire.lti.core.business.commands.InitiateOidcCommand;
import riomhaire.lti.core.business.commands.LTILaunchCommand;
import riomhaire.lti.core.business.commands.LaunchContentCommand;
import riomhaire.lti.core.infrastructure.facades.clientregistration.RedisBasedClientRegistrationResolver;
import riomhaire.lti.core.model.LTIMessage;
import riomhaire.lti.core.model.LTIMessageType;
import riomhaire.lti.core.model.interfaces.ApplicationRegistry;
import riomhaire.lti.core.model.interfaces.ClientRegistrationResolver;
import riomhaire.lti.core.model.interfaces.CommandDispatcher;

/**
 * This is the main entry point for the LTI Tool and well as salient configuration of core important items.
 *
 * In this simplified implementation we have:
 *
 *  [Application Registry] Which contains information about environment and services the commands can execute - everything is in one place.
 *       -
 */
@SpringBootApplication
@Configuration
public class JavaLti13ToolApplication {

    /**
     * This is the entry point - eveything starts here
     * @param args command line arguments
     */
    public static void main(String[] args) {
        SpringApplication.run(JavaLti13ToolApplication.class, args);
    }

    /**
     * This builds the implementation of the service which allows us to look up a deployed and registered tool within
     * our system by default we store a configuration within redis ... see src/redis folder for an example
     * @param template this is the template engine which allows us to interact with redis - provided by spring
     * @return ClientRegistrationResolver which allows us to look up a client registration based on certain parameters
     */
    @Bean
    ClientRegistrationResolver buildClientRegistrationResolver(RedisTemplate<String, String> template) {
        return new RedisBasedClientRegistrationResolver(template);
    }

    /**
     * This Spring bean creates the command dispatcher and populates it for the current tool .
     * It is here where you map messageType to command which handles it
     *
     * Currently defined messages are:
     *
     *   - "ProcessOidcRequest"      : This is the command launched when the 'initiate' oidc launch command
     *   - "ProcessLTIMessage"       : LTI Launch-content/deep-link are contained within a JWT.. this command
     *                                 decodes the JWT and delegates to 'LaunchContent', 'SelectDeepLinkContent' etc
     *   - "LaunchContent"           : This is the message associated with launch a basic resource
     *   - "SelectDeepLinkContent"   : This is the message sent to handle 'we need to select content for a deep link'
     *
     * @return the Spring View-Model result of applyings a command to a message
     */
    @Bean
    CommandDispatcher<ModelAndView, ApplicationRegistry<ModelAndView,LTIMessage>, LTIMessage> buildCommandDispatcher() {
        return new LTIMessageDispatcher()
                .addCommand(LTIMessageType.ProcessOidcRequest.name(), new InitiateOidcCommand())
                .addCommand(LTIMessageType.ProcessLTIMessage.name(), new LTILaunchCommand())
                .addCommand(LTIMessageType.LaunchContent.name(), new LaunchContentCommand());

    }

    /**
     * This builds the registry for this tool/application
     *
     * @param commandDispatcher            Dispatches a message to an appropriate command
     * @param clientRegistrationResolver   Look up the configuration for a particular client
     * @return Registry            environment links  we execute within
     */
    @Bean
    public Registry buildRegistry(@NonNull CommandDispatcher<ModelAndView, ApplicationRegistry<ModelAndView,LTIMessage>, LTIMessage> commandDispatcher, @NonNull ClientRegistrationResolver clientRegistrationResolver) {
        return new Registry(commandDispatcher, clientRegistrationResolver);
    }

}
