package riomhaire.lti.core.business.commands;

import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.view.RedirectView;
import org.springframework.web.servlet.view.json.MappingJackson2JsonView;
import riomhaire.lti.core.model.LTIMessage;
import riomhaire.lti.core.model.interfaces.ApplicationRegistry;
import riomhaire.lti.core.model.interfaces.Command;

import java.util.UUID;

/**
 * Defines  execute command on message P within environment E returning T
 *
 * @param <T> What object is returned by the command
 * @param <E> WHat execution environment is to be used (usually some form of Application Registry)
 * @param <P> The message the command can process
 *
 * This implementation is for the OIDC Initiation call made by an issuer as part of the security specification.
 */
@AllArgsConstructor
@Slf4j
public class InitiateOidcCommand implements Command<ModelAndView, ApplicationRegistry<ModelAndView,LTIMessage>, LTIMessage> {


    public ModelAndView execute(ApplicationRegistry<ModelAndView,LTIMessage> registry, LTIMessage message) {
        final ModelAndView mav;
        var queryParams = message.getQueryParams();
        String iss = queryParams.get("iss");
        String clientId = queryParams.get("client_id");
        var clientRegistration = registry.clientRegistrationResolver().lookupClient(iss, clientId);

        if (clientRegistration.isPresent()) {
            // Build Location
            StringBuilder location = new StringBuilder();
            var clientConfiguration = clientRegistration.get();
            var loginHint = queryParams.get("login_hint");
            var ltiMessageHint = queryParams.get("lti_message_hint");
            var targetLinkUri = queryParams.get("target_link_uri");


            location.append(clientConfiguration.getAuthenticateUrl());
            location.append("?");
            location.append("scope=openid&response_type=id_token&client_id=");
            location.append(clientConfiguration.getClientId());
            location.append("&login_hint=");
            location.append(loginHint);

            // This should be used to map to some backend thing we can verify ... jwt etc
            location.append("&state=");
            final String state = UUID.randomUUID().toString();
            location.append(state);

            // This should be used to map to some backend thing we can verify ... as once only
            location.append("&response_mode=form_post&prompt=none&nonce=");
            location.append(UUID.randomUUID().toString());
            location.append("&lti_message_hint=");
            location.append(ltiMessageHint);
            location.append("&redirect_uri=");
            location.append(targetLinkUri);

            log.info("OIDC Redirecting to " + location.toString());
            mav = new ModelAndView(new RedirectView(location.toString()));

        } else {
            String msg = "unknown issuer [" + iss + "] client [" + clientId + "]";
            log.error(msg);
            MappingJackson2JsonView view = new MappingJackson2JsonView();
            view.setPrettyPrint(true);
            mav = new ModelAndView(view);
            mav.addObject("error", msg);
            mav.addObject("queryParams", queryParams);
        }
        return mav;

    }
}
