package riomhaire.lti.core.infrastructure.facades.clientregistration;

import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.SneakyThrows;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.util.DigestUtils;
import riomhaire.lti.core.model.ClientConfiguration;
import riomhaire.lti.core.model.interfaces.ClientRegistrationResolver;

import java.util.Optional;


/**
 *  When a tool/partner-lms is registered we need to store this information somewhere ... and we have
 *  abstracted this out as a  'ClientRegistrationResolver'. Implementations of this interface  could be
 *  to a file, DB or Redis - but all the call wants to know about is I want to find out about this deployment
 *
 *  This implementation is for Redis
 */
public class RedisBasedClientRegistrationResolver implements ClientRegistrationResolver {
    final ObjectMapper objectMapper = new ObjectMapper();
    private final RedisTemplate<String, String> template;

    /**
     *
     * @param template THe spring redis template
     */
    public RedisBasedClientRegistrationResolver(RedisTemplate<String, String> template) {
        this.template = template;
    }

    /**
     * Return the registration information related to this issuer/client-id pair
     * @param issuer    ... This is canvas/blackboard or some other entity
     * @param clientId  ... This is a particular tool within the issuer environment
     * @return  ClientConfiguration if found
     */
    @SneakyThrows
    @Override
    public Optional<ClientConfiguration> lookupClient(String issuer, String clientId) {
        var hash = DigestUtils.md5DigestAsHex((issuer + clientId).getBytes());
        var key = String.format("lti-client-registration-%s", hash);
        var blob = template.opsForValue().get(key);

        if (blob != null) {
            ClientConfiguration client = objectMapper.readValue(blob, ClientConfiguration.class);
            return Optional.of(client);
        }
        return Optional.empty();
    }
}
